# FR-2: File Partitioning & Chunk Management

## 1. Overview

This document outlines the detailed requirements for partitioning files into manageable chunks and the associated management of these chunks during storage and retrieval. The aim is to optimize file transfer, deduplication, and versioning while ensuring that the file reassembly process maintains data integrity even under adverse conditions such as simulated network or disk latency and failures.

## 2. Objectives & Scope

### 2.1 Objectives

- **Efficient Upload & Download:** Enable efficient file transfers by breaking large files into smaller chunks.
- **Deduplication:** Support content-addressable storage by indexing each chunk via a cryptographic hash (e.g., SHA-256) to allow redundant data detection and avoidance.
- **Resilient Reassembly:** Ensure that files can be accurately reassembled from individual chunks in the correct order, preserving the original file's integrity.
- **Performance Under Failure Conditions:** Validate that the chunking mechanism performs robustly even during simulated network and disk failure scenarios.

### 2.2 Scope

- Applies to all file operations where files are partitioned before upload.
- Covers algorithms to partition files, assign identifiers, and maintain metadata for reassembly.
- Includes supporting tools for integrity checking and deduplication.
- Integrates with deterministic simulation testing to simulate adverse conditions during chunk processing.

## 3. Detailed Requirements

### 3.1 File Partitioning Strategy

- **Chunking Approach:**

  - Files must be divided into discrete segments (chunks) either using a fixed-size approach (for example, 1 MB per chunk) or a content-defined boundary method.
  - The chosen approach should favor predictability in reassembly while balancing performance and effective deduplication.
  - Implementation should remain flexible, with the possibility to choose fixed-size chunking during initial deployment and exploring content-defined methods later if necessary.

- **Metadata Association:**
  - Each chunk shall be assigned metadata, including:
    • A unique identifier derived from a cryptographic hash (e.g., SHA-256) computed on the plaintext of the chunk.
    • Sequence information to allow correct reassembly.
    • Timestamps and any additional markers for version history.
  - The metadata must be stored alongside the encrypted chunk ensuring the data’s lineage and integrity during recovery.

### 3.2 Management of Chunked Data

- **Encryption and Indexing of Chunks:**

  - Each file chunk is to be encrypted individually on the client side before transmission.
  - Post encryption, a cryptographic hash is computed (or preserved from pre-encryption, if applicable) to serve as a unique identifier for deduplication and integrity verification.
  - The encryption must ensure that even if two identical chunks occur (or near-identical), the system can recognize and avoid storing duplicates.

- **Chunk Deduplication:**
  - The system shall implement a deduplication mechanism whereby if a chunk with an identical hash already exists within the system, the chunk is not stored a second time.
  - When a duplicate is detected, the system should update its reference count or metadata to indicate reuse, reducing storage overhead.

### 3.3 Reassembly of Files

- **Reassembly Process:**

  - A complete file must be reconstructible by sequentially fetching each required chunk based on its stored metadata and then decrypting and concatenating them in their correct order.
  - Reassembly operations must include integrity verification, ensuring that each chunk matches its expected hash before being integrated into the final output.

- **Error Handling during Reassembly:**
  - In the event of a missing or corrupted chunk, the system must trigger a re-request from alternative nodes (if replication is available) and retry the operation.
  - The user must be notified if reassembly fails after all recovery attempts, and logs should capture the details of the error for auditability.

### 3.4 Robustness Under Simulated Adverse Conditions

- **Deterministic Simulation Integration:**

  - The chunking and reassembly processes must integrate with the deterministic simulation testing framework, allowing testers to simulate:
    • Network latency and intermittent connectivity during chunk upload/download.
    • Disk I/O delays or failures during chunk read/write operations.
  - These simulations must be repeatable, ensuring that robustness and error handling can be thoroughly validated in controlled test scenarios.

- **Timeouts and Retries:**
  - In case of errors during chunk transfer, the system must implement appropriate retry logic and timeouts.
  - Retried operations should ensure consistency and integrity checks are reapplied to avoid propagating errors.

### 3.5 Performance Considerations

- **Optimized Partitioning:**

  - The chunking algorithm should be highly efficient to minimize processing overhead, even for large files.
  - Use of multithreading or asynchronous processing shall be considered to achieve higher throughput during chunking, encryption, and uploading.

- **Resource Management:**
  - The system must monitor and regulate the resource usage (CPU, memory, disk I/O) during the process of partitioning and reassembly.
  - Built-in diagnostic tools should measure performance metrics (e.g., chunk processing time, deduplication success rates) and enable adjustments to the configuration parameters dynamically.

## 4. Measurable Criteria & Test Cases

### 4.1 Automated Tests

- **File Chunking and Reassembly Test:**

  - Create test files of varying sizes.
  - Verify that the file is partitioned into chunks of the expected size.
  - Ensure that upon download and correct decryption, the reassembled file’s content exactly matches the original using byte-for-byte comparisons (via cryptographic hash comparison).

- **Deduplication Validation Test:**

  - Upload files with repeated content.
  - Verify that identical chunks are recognized and stored only once while references to the duplicate are maintained.

- **Simulated Failure Conditions Test:**
  - Run tests with aggressive simulated network and disk latencies or intentional failures during chunk processing.
  - Confirm that robust retry logic successfully fetches all chunks and that error paths are adequately logged.

### 4.2 Performance Benchmarks

- **Chunk Processing Speed:**

  - Measure the average time required for file partitioning and reassembly across different file sizes.
  - Validate that these times meet the performance targets as defined in the broader NFR-2 (Performance & Responsiveness) requirements.

- **Resource Utilization:**
  - Monitor CPU, memory, and disk I/O usage during extensive chunk processing tasks.
  - Compare against baseline performance profiles to ensure that the process scales efficiently, even under high load scenarios.

## 5. Dependencies & Integration Points

- **Encryption Module:**

  - Fully integrated with the encryption procedures defined in FR-1 and ADR-0002, ensuring that each chunk is securely encrypted.

- **Manifest Management:**

  - The chunk metadata and ordering information is recorded in the immutable, append-only manifest as described in ADR-0006. This ensures a verifiable history for file reassembly and version control.

- **Deterministic Simulation Framework:**
  - Integration with the simulation framework is critical to ensure that the chunking strategy is robust under adverse conditions (refer to the deterministic simulation aspects in FR-1).

## 6. Security Considerations

- **Data Confidentiality:**

  - Ensure that no sensitive plaintext is written to disk during or after the chunking process.

- **Integrity and Auditability:**

  - All operations on chunks (creation, encryption, deduplication, assembly) must be logged in a tamper-evident manner, supporting the overall auditability requirements of the system.

- **Fallback Mechanisms:**
  - Consider secure fallback strategies if one type of partitioning algorithm fails or does not perform adequately under certain environmental conditions.

## 7. Conclusion

FR-2 establishes the core mechanism for partitioning files into chunks, ensuring robust management for efficient uploads and downloads in a secured environment. By combining efficient, flexible chunking with strong deduplication, metadata tracking, and robust reassembly mechanisms, this requirement supports the broader system goals of performance, scalability, and data integrity even under simulated failure conditions.
